import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter_gen/gen_l10n/l10n.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:process_run/shell.dart';
import 'package:flutter_demo_experience/const/colors.dart';
import 'package:flutter_demo_experience/const/dimens.dart';
import 'package:flutter_demo_experience/json_demo.dart';
import 'package:flutter_demo_experience/provider.dart';
import 'package:flutter_demo_experience/widget/armadillo_icon.dart';

class DemoLaunch extends ConsumerWidget {
  const DemoLaunch({super.key});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final Size widgetSize = MediaQuery.of(context).size;
    final isExecutableDemo = ref.watch(isExecutableDemoProvider);
    final demoRoute = ref.watch(demoRouteProvider);
    final langButton = ref.watch(langButtonProvider);
    AsyncValue<JsonModel> jsonModel = ref.watch(jsonModelProvider);

    if (isExecutableDemo) {
      return jsonModel.when(
          data: (jsonModel) {
            final Map<String, dynamic> demoJson =
                _selectedDemoJson(jsonModel, demoRoute);

            return Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                _demoNameWidget(demoJson['name'], widgetSize),
                langButton.first
                    ? _demoDescriptionWidget(
                        demoJson['jaDescription'], widgetSize)
                    : _demoDescriptionWidget(
                        demoJson['enDescription'], widgetSize),
                Expanded(
                  child: _demoIconWidget(demoJson['screenshot']),
                ),
                _demoLaunchButtonWidget(
                    context, ref, demoJson['executable'], widgetSize)
              ],
            );
          },
          error: (error, stackTrace) => Text('Error: $error'),
          loading: () => const CircularProgressIndicator());
    } else {
      return const ArmadilloIcon();
    }
  }
}

Widget _demoNameWidget(String title, Size widgetSize) {
  final double fontSize = widgetSize.width * demoTitleFontSizeRatio;
  return Text(
    title,
    style: TextStyle(
        fontWeight: FontWeight.bold, fontSize: fontSize, color: textColor),
  );
}

Widget _demoIconWidget(String iconName) {
  return Image.asset(
    'assets/screenshot/$iconName',
    height: 200,
  );
}

Widget _demoDescriptionWidget(String description, Size widgetSize) {
  final double fontSize = widgetSize.width * demoDescriptionFontSizeRatio;
  return Text(
    description,
    style: TextStyle(fontSize: fontSize, color: textColor),
  );
}

Widget _demoLaunchButtonWidget(
    BuildContext context, WidgetRef ref, String executable, Size widgetSize) {
  final bool isExecution = ref.watch(isExecutionProbider);
  final Shell shell = ref.watch(shellProvider);
  final double fontSize = widgetSize.width * demoLaunchFontSizeRatio;
  final String buttonText =
      isExecution ? L10n.of(context)!.finish : L10n.of(context)!.launch;
  String execFilePath;
  return ElevatedButton(
      // Exception handling because the app will crash if there is no file to execute
      onPressed: () {
        // TFLite demo has a path after python3.
        if (executable.split(' ')[0] == 'python3') {
          execFilePath = executable.split(' ')[1];
        } else {
          execFilePath = executable;
        }
        try {
          if (!File(execFilePath).existsSync()) {
            throw ProcessException(execFilePath,
                ["Cannot execute because the file does not exist"]);
          }
          isExecution ? shell.kill() : shell.run(executable);
          ref.watch(shellProvider.notifier).state = shell;
          ref.watch(isExecutionProbider.notifier).update((state) => !state);
        } on ProcessException catch (e) {
          // Display messages in terminal and snackbar if execution path does not exist
          final snackBar = SnackBar(content: Text('$e'));
          debugPrint('$e');
          ScaffoldMessenger.of(context).showSnackBar(snackBar);
        }
      },
      child: Text(buttonText,
          style: TextStyle(fontSize: fontSize, color: textColor)));
}

Map<String, dynamic> _selectedDemoJson(
    JsonModel jsonModel, List<String> demoRoute) {
  Map<String, dynamic> demoJson = jsonModel.toJson();
  List<Map<String, dynamic>> demoJsonList = [];

  demoJson = demoJson[demoRoute.first];
  demoJsonList = demoJson[demoRoute[1]];
  final Map<String, dynamic> json = demoJsonList
      .singleWhere((element) => element.containsValue(demoRoute.last));

  return json;
}
