import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_demo_experience/json_demo.dart';
import 'package:flutter_demo_experience/provider.dart';
import 'package:flutter_demo_experience/const/colors.dart';
import 'package:flutter_demo_experience/const/dimens.dart';

class DemoList extends ConsumerWidget {
  final Size size;
  const DemoList(
      {Key? key, this.size = const Size(double.infinity, double.infinity)})
      : super(key: key);

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    AsyncValue<JsonModel> jsonModel = ref.watch(jsonModelProvider);

    return jsonModel.when(
        data: (jsonModel) {
          return SizedBox(
            width: size.width,
            height: size.height,
            child: _demoMenuWidget(ref, jsonModel, size),
          );
        },
        error: (error, stackTrace) => Text('Error: $error'),
        loading: () => const CircularProgressIndicator());
  }
}

Widget _demoMenuWidget(WidgetRef ref, JsonModel jsonModel, Size widgetSize) {
  final List<String> demoRoute = ref.watch(demoRouteProvider);
  final bool isExecutableDemo = ref.watch(isExecutableDemoProvider);
  List<String> titles = ref.watch(displayTitleProvider);
  List<String> imagePaths = ref.watch(imagePathsProvider);
  bool isNextExecutableDemo = false;

  if (!isExecutableDemo) {
    isNextExecutableDemo = _nextExecutableDemoIsCheck(jsonModel, demoRoute);
    titles = _getDemoTitle(jsonModel, demoRoute);
  }
  if (isNextExecutableDemo) {
    imagePaths = _getExecutableDemoInfo(jsonModel, demoRoute, 'screenshot');
  }

  return (isExecutableDemo || isNextExecutableDemo)
      ? _demoMenuWithScreenshot(ref, titles, imagePaths, widgetSize)
      : _demoMenu(ref, titles, widgetSize);
}

Widget _demoMenuWithScreenshot(WidgetRef ref, List<String> titles,
    List<String> imagePaths, Size widgetSize) {
  final isExecutableDemo = ref.watch(isExecutableDemoProvider);
  final fontSize = widgetSize.width * executableDemoListFontSizeRatio;
  final List<Widget> items = [];

  for (int i = 0; i < titles.length; i++) {
    final Widget item = Padding(
      padding: const EdgeInsets.all(menuItemsWithImagePadding),
      child: ElevatedButton(
        onPressed: () {
          ref.watch(displayTitleProvider.notifier).state = titles;
          ref.watch(imagePathsProvider.notifier).state = imagePaths;
          // Replace the end of route if you click on successive executable demos
          ref.watch(demoRouteProvider.notifier).update(
            (route) {
              if (isExecutableDemo) route.removeLast();
              route.add(titles[i]);
              return [...route];
            },
          );
          ref.watch(isExecutableDemoProvider.notifier).state = true;
        },
        style: ElevatedButton.styleFrom(backgroundColor: bodyBackgroundColor),
        child: Column(
          children: [
            Expanded(
              child: Image.asset('assets/screenshot/${imagePaths[i]}'),
            ),
            Text(
              titles[i],
              style: TextStyle(fontSize: fontSize, color: textColor),
            )
          ],
        ),
      ),
    );
    items.add(item);
  }
  return GridView.count(
    crossAxisCount: 2,
    children: items,
  );
}

Widget _demoMenu(WidgetRef ref, List<String> titles, Size widgetSize) {
  final fontSize = widgetSize.width * demoListFontSizeRatio;
  final List<Widget> items = [];

  for (final String title in titles) {
    final Widget item = Padding(
      padding: const EdgeInsets.all(menuItemsPadding),
      child: ElevatedButton(
        onPressed: () {
          ref.watch(displayTitleProvider.notifier).state = titles;
          ref.watch(demoRouteProvider.notifier).update(
            (route) {
              route.add(title);
              return [...route];
            },
          );
        },
        style: ElevatedButton.styleFrom(backgroundColor: bodyBackgroundColor),
        child: Text(
          title,
          style: TextStyle(fontSize: fontSize, color: textColor),
        ),
      ),
    );
    items.add(item);
  }
  return ListView(
    children: items,
  );
}

bool _nextExecutableDemoIsCheck(JsonModel jsonModel, List<String> demoRoute) {
  bool isNextExecutableDemo = false;
  Map<String, dynamic> demoJson = jsonModel.toJson();

  if (demoRoute.isNotEmpty) {
    for (int i = 0; i <= demoRoute.length - 1; i++) {
      if (demoJson[demoRoute[i]] is List<Map<String, dynamic>>) {
        isNextExecutableDemo = true;
      } else {
        demoJson = demoJson[demoRoute[i]];
      }
    }
  }
  return isNextExecutableDemo;
}

List<String> _getExecutableDemoInfo(
    JsonModel jsonModel, List<String> demoRoute, String info) {
  Map<String, dynamic> demoJson = jsonModel.toJson();
  List<String> infoList = [];
  for (int i = 0; i <= demoRoute.length - 1; i++) {
    if (demoJson[demoRoute[i]] is List<Map<String, dynamic>>) {
      demoJson[demoRoute[i]].forEach((element) => infoList.add(element[info]));
    } else {
      demoJson = demoJson[demoRoute[i]];
    }
  }
  return infoList;
}

List<String> _getDemoTitle(JsonModel jsonModel, List<String> demoRoute) {
  Map<String, dynamic> demoJson = jsonModel.toJson();
  List<String> demoList = [];

  if (demoRoute.isEmpty) {
    demoList = demoJson.keys.toList();
  } else {
    for (int i = 0; i <= demoRoute.length - 1; i++) {
      // If List comes in, display the Execute button.
      if (demoJson[demoRoute[i]] is List<Map<String, dynamic>>) {
        demoList = [];
        demoJson[demoRoute[i]]
            .forEach((element) => demoList.add(element['name']));
      } else {
        demoJson = demoJson[demoRoute[i]];
        demoList = demoJson.keys.toList();
      }
    }
  }
  return demoList;
}
